.TH XXALLOC 3 local
.SH NAME
xxalloc \- family of routines for multi\-dimensional dynamic array allocation,
initialization, printing, renumbering, and freeing
.SH SYNOPSIS
.sp
.B #include <xxalloc.h>
.br
.B cc
[flags] files
.B -lxxalloc [-lxxerror]
[libraries]
.sp
.B
Base Routines
.sp
void xx_error(s)
char *s;
.br
char *xx_alloc(n)
unsigned n;
.br
void xx_free(p, n)
char *p;
unsigned n;
.br
long xx_amount()
.sp
.B
Prefixes and Arguments
.sp
{type} \- char \- short \- long \- int \- float \- double
.br
{*} \- * \- ** \- *** for 1 \- 2 \- 3 dimensions
.br
{t} \- c \- s \- l \- i \- f \- d
.br
{d} \- 1 \- 2 \- 3
.br
{args} \- ib,ie \- ib,ie,jb,je \- ib,ie,jb,je,kb,ke
.br
.sp
.B
Functions for Simple Types
.sp
{type} {*}{t}{d}_alloc({args})
.br
int {args};
.sp
void {t}{d}_init(m, {args}, ini)
.br
{type} {*}m;
.br
int {args};
.br
{type} ini;
.sp
void {t}{d}_prnt(fp, fmt, m, {args})
.br
FILE *fp;
.br
char *fmt;
.br
{type} {*}m;
.br
int {args};
.sp
{type} {*}{t}{d}_renum(m, {args}, {args})
.br
{type} {*}m;
.br
int {args}, {args};
.sp
void {t}{d}_free(m, {args})
.br
{type} {*}m;
.br
int {args};
.sp
.bp
.B
Functions for Structures
.sp
char {*}n{d}_alloc({args}, n)
.br
int {args}, n;
.sp
void n{d}_init(m, {args}, n, nini)
.br
char {*}m;
.br
int {args}, n;
.br
void (*nini)();
.sp
void n{d}_prnt(fp, nprn, m, {args}, n)
.br
FILE *fp;
.br
void (*nprn)();
.br
char {*}m;
.br
int {args}, n;
.sp
char {*}n{d}_renum(m, {args}, {args}, n)
.br
char {*}m;
.br
int {args}, {args}, n;
.sp
void n{d}_free(m, {args}, n)
.br
char {*}m;
.br
int {args}, n;
.sp
.SH DESCRIPTION
.PP 
xxalloc is a family of routines for dynamic array manipulation in one,
two and three dimensions.  Routines are included for allocation,
initialization, printing, renumbering, and freeing both arrays of
structures and arrays of simple types.  Since the "edge-vector" approach
is used for two and three dimensional arrays, this set of routines
allows for the development of reusable subroutine libraries without
regard to some "maximum" dimension.  Both positive and negative indices
are allowed.  In this document "array of" is often used where "pointer
to" would be more correct. 
.PP
.B
Base Routines
.IP "xx_error"
The required error handling routine. The default routine exits
when called, but it may be replaced by a user's routine of the same name.
The default routine may be linked in by
.IP
.B cc
[flags] files
.B \-lxxalloc \-lxxerror
[libraries]
.sp
The default routine is simply
.sp
#include <stdio.h>
.br
void xx_error(s)
.br
char *s;
.br
{
.br
	fprintf(stderr, s);
.br
	exit(2);
.br
	return;
.br
}
.IP "xx_alloc"
Allocates memory and checks for error return.  Similar to malloc().
Returns a NULL pointer if there is no available memory or another error occurs.
.IP "xx_free"
Free memory allocated with xx_alloc(). Similar to free() but take an extra
argument for the amount of memory.
.IP "xx_amount"
Returns the amount of memory currently allocated with this package.
.PP
.B
The suffix on the function name indicates its function.
.B
The five function types are
.IP "_alloc"
Allocate memory.  Returns an appropriate NULL pointer if there is no
available memory or another error occurs. 
.IP "_init"
Initialize.
.IP "_prnt"
Print.
.IP "_renum"
Renumber the array indices.
.IP "_free"
Free the allocated memory.
.PP
.B
The prefix on the function name indicates its type. They are
.PP
.IP "{type}"
The data type the function manipulates.
.IP
char \- array of structure
.br
char \- array of char
.br
short \- array of short
.br
long \- array of long
.br
int \- array of int
.br
float \- array of float
.br
double \- array of double
.PP
.IP "{*}"
Number of dimensions
.IP
* \- 1 dimension \- pointer to 
.br
** \- 2 dimensions \- pointer to pointer to
.br
*** \- 3 dimensions \- pointer to pointer to pointer to
.PP
.IP "{t}"
The type that the function manipulates.
.IP
n \- array of structure
.br
c \- array of char
.br
s \- array of short
.br
l \- array of long
.br
i \- array of int
.br
f \- array of float
.br
d \- array of double
.PP
.IP "{d}"
The dimension of the array to be manipulated. 
.IP
1 \- one dimension
.br
2 \- two dimensions
.br
3 \- three dimensions
.PP
.B
Arguments common to both functions that operate on simple types and structures.
.PP
.IP "{args}"
Integer arguments specifying the beginning and ending indices of each
dimension of the array.  For example the length of an array with
ib = \-2 and ie = 5 would be ((5 \- (-2)) + 1).
.IP 
ib, ie \- row indices
.br
ib, ie, jb, je \- row and column indices
.br
ib, ie, jb, je, kb, ke \- row, column, and third dimension indices
.PP
.IP "fp"
FILE pointer for the printing functions' output.
.PP
.B
Arguments to functions that operate on simple types.
.PP
.IP "ini"
Value to initialize the array to.  Must be of {type}.
.PP
.IP "fmt"
Character string to be passed as an argument to fprintf.  It should
contain the conversion specification for a single argument of {type}. 
.PP
.B
Arguments to functions that operate on structures.
.PP
.IP "n"
sizeof the structure.
.PP
.IP "nini"
Pointer to a function used to initialize the structure.  The function
receives a pointer to the structure as the only argument as shown.
.sp
void initialize_some_structure(some_struct)
.br
struct *some_struct;
.PP
.IP "nprn"
Pointer to a function used to print the structure.  The function
receives a FILE pointer and a pointer to the structure as arguments as shown.
An example is
given in the examples section.
.sp
void print_some_structure(fp, some_struct)
.br
FILE *fp;
.br
struct *some_struct;
.SH EXAMPLES
.sp
.B
Example1 \- Operations on a two dimensional array of doubles
.sp
/* example1.c */
.sp
#include <stdio.h>
.br
#include <xxalloc.h>
.sp
void mtxtsp(a, ib, ie, jb, je, b)
.br
double **a, **b;
.br
int ib, ie, jb, je;
.br
{
.br
	int i, j;
.br
	double tmp;
.br
	
.br
	for (i = ib; i <= ie; i++)
.br
 		for (j = i; j <= je; j++)
.br
		{
.br
			tmp = a[i][j];
.br
			b[i][j] = a[j][i];
.br
			b[j][i] = tmp;
.br
		}
.br
	return;
.br
}
.sp
main()
.br
{
.br
	double **a;
.br
	int i, j;
.br
	
.br
/* allocate a two dimensional array of doubles */
.sp
	a = d2_alloc(0, 9, 0, 9);
.sp
/* initialize the array to 1.0 and print */
.sp
	d2_init(a, 0, 9, 0, 9, 1.0);
.br
	printf("\\ninitialized to 1\\n");
.br
	d2_prnt(stdout, "%5.0f ", a, 0, 9, 0, 9);
.sp
/* set the array equal to the 2*i + j and print */
.sp
	for (i = 0; i < 10; i++)
.br
		for (j = 0; j < 10; j++)
.br
			a[i][j] = (double) (2*i + j);
.br
	printf("\\n2*i + j\\n");
.br
	d2_prnt(stdout, "%5.0f ", a, 0, 9, 0, 9);
.sp
/* take the transpose and print*/
.sp
	mtxtsp(a, 0, 9, 0, 9, a);
.br
	printf("\\ntransposed\\n");
.br
	d2_prnt(stdout, "%5.0f ", a, 0, 9, 0, 9);
.sp
/* renumber the indices a la FORTRAN and print */
.br
/* the new indices must be used from now on */
.sp
	a = d2_renum(a, 0, 9, 0, 9, 1, 10, 1, 10);
.br
	printf("\\nrenumbered\\n");
.br
	d2_prnt(stdout, "%5.0f ", a, 1, 10, 1, 10);
.sp
/* renumber the indices a la C and print */
.br
/* the new indices must be used from now on */
.sp
	a = d2_renum(a, 1, 10, 1, 10, 0, 9, 0, 9);
.br
	printf("\\nrenumbered again\\n");
.br
	d2_prnt(stdout, "%5.0f ", a, 0, 9, 0, 9);
.sp
/* free the allocated memory */
.sp
	d2_free(a, 0, 9, 0, 9);
.br
	exit(0);
.br
}		
.sp
.B
Example 2 \- Operations on an array of structures
.sp
/* example2.c */
.sp
#include <stdio.h>
.br
#include <xxalloc.h>
.sp
typedef struct
.br
{
.br
	double dnum;
.br
	int inum;
.br
} NUM;
.sp
/* structure initialization function */
.br
/* passed as an argument to {d}n_init() */
.br
void num_init(num)
.br
NUM *num;
.br
{
.br
	num->dnum = 1.0;
.br
	num->inum = 1;
.br
	return;
.br
}
.sp
/* structure printing function */
.br
/* passed as an argument to {d}n_prnt() */
.br
void num_prnt(fp, num)
.br
FILE *fp;
.br
NUM *num;
.br
{
.br
	fprintf(fp, "%5.1f %3d ", num->dnum, num->inum);
.br
	return;
.br
}
.sp
main()
.br
{
.br
	int i;
.br
	NUM *num;
.br
	
.br
/* allocate a one dimensional array of NUM */
.sp
	num = (NUM *) n1_alloc(0, 4, sizeof(NUM));
.sp
/* initialize the array to 1.0 and print */
.sp
	n1_init(num, 0, 4, sizeof(NUM), num_init);
.br
	printf("\\ninitialized to 1\\n");
.br
	n1_prnt(stdout, num_prnt, num, 0, 4, sizeof(NUM));
.sp
/* set the array equal to the i and print */
.sp
	for (i = 0; i <= 4; i++)
.br
	{
.br
		num[i].dnum = (double) i;
.br
		num[i].inum = i;
.br
	}
.br
	printf("\\nset to i\\n");
.br
	n1_prnt(stdout, num_prnt, num, 0, 4, sizeof(NUM));
.sp
/* renumber the indices a la FORTRAN and print */
.br
/* the new indices must be used from now on */
.sp
	num = (NUM *) n1_renum(num, 0, 4, 1, 5, sizeof(NUM));
.br
	printf("\\nindices renumbered\\n");
.br
	n1_prnt(stdout, num_prnt, num, 1, 5, sizeof(NUM));
.br
	printf("\\n");
.sp
/* free the allocated memory */
.sp
	n1_free(num, 1, 5, sizeof(NUM));
.br
	exit(0);
.br
}
.SH FILES
.PP
/lib/xxalloc.a
.br
/lib/xxerror.a
.SH BUGS
.PP
There is only minimal error checking and no error checking at all
between function calls.
.SH AUTHOR
.PP
Harold G. Walters

